#!/usr/bin/env python3

from log_parser import LogParser, LOGGER

if __name__ == "__main__":
    # log
    LOGGER.debug("debug log")
    LOGGER.info("info log")
    LOGGER.ok("ok log")
    LOGGER.warning("warning log")
    LOGGER.error("error log")
    LOGGER.critical("critical log")
    # log_parser
    file_handlers = LOGGER.get_fileHandlers()
    for file_handler in file_handlers:
        print(f"Parse logfile : '{file_handler.baseFilename}'")
        log_parser = LogParser(file_handler.baseFilename) #@TODO  parser  Logger plutot qu'individuel
        regex =  r"([\d-]+\s[\d:]+,[\d:]+)\s\[\s*([A-Z]+)\]\s*([A-Za-z]+.[A-Za-z]+):(\d+)\s*\|*(.*)" #@TODO conf
        log_parser.load(file_handler,regex)
        log_parser.stdout(LOGGER.get_streamHandlers()[0])
    


#!/usr/bin/env python3
from datetime import datetime
from pathlib import Path
import re
import logging
from log import Logger

LOGGER = Logger()


class LogParser:
    def __init__(self, logfile_path: str):
        self.logfile_path: str = logfile_path
        self.logs = []

    def is_logfile_exists(self):
        """is logfile existing

        Returns:
            bool: file exist
        """
        return Path(self.logfile_path).exists()

    def logfile_exists(self):
        """check if logfile exists"""
        if not self.is_logfile_exists():
            LOGGER.error(f"'{self.logfile_path}' not found")
            exit()

    def dump(self):
        """dump logfile"""
        Path(self.logfile_path).unlink(missing_ok=True)

    @staticmethod
    def _string_to_LogRecord(string: str, file_handler: logging.FileHandler, regex: str):

        found = re.search(regex, string)
        if file_handler.formatter:
            date_fmt = (
                file_handler.formatter.default_time_format
                if file_handler.formatter.datefmt == ""
                else file_handler.formatter.datefmt
            )
            date_fmt += (
                ",%f" if file_handler.formatter.default_msec_format else ""
            )  # @TODO nok le %f pas vraiment exacte

            if found and len(found.groups()) == 5:
                log = logging.LogRecord(
                    name="",
                    level=logging.getLevelName(found.group(2)),
                    pathname=found.group(3),
                    lineno=int(found.group(4)),
                    msg=found.group(5),
                    args=None,
                    exc_info=None,
                )

                t = datetime.strptime(found.group(1), date_fmt)  # type: ignore
                log.created = t.timestamp()

                return log

    def load(self, file_handler: logging.FileHandler, regex: str):
        self.logfile_exists()
        logfile = open(self.logfile_path, "r")
        logfile_lines = logfile.readlines()

        # Check if regex is valid with this FileHandler
        log_sample = file_handler.format(logging.LogRecord("name", logging.INFO, "pathname", 0, "message", None, None))
        check = re.search(regex, log_sample)

        if check is None or len(check.groups()) != 5:
            LOGGER.error(
                f"regex specified '{regex}' is not valid for FileHandler '{file_handler.name}' for '{self.logfile_path}'"
            )
            exit()

        # Build LogRecord
        self.logs = []
        for i in range(len(logfile_lines)):
            log = self._string_to_LogRecord(
                logfile_lines[i],
                file_handler,
                regex,
            )

            # format the log correctly (avoid line breaking)
            if isinstance(log, logging.LogRecord):
                self.logs.append(log)
            else:
                if len(self.logs) > 0:
                    last = self.logs.pop()
                    if isinstance(last, logging.LogRecord):
                        last.msg += f"\n{logfile_lines[i]}"
                        self.logs.append(last)

    def stdout(self, stream_handler: logging.StreamHandler):
        """print log to stdout

        Args:
            stream_handler (logging.StreamHandler) : stream handler to use convert logs
        """
        index = 0

        for log in self.logs:
            index += 1
            if isinstance(log, logging.LogRecord):
                print(f"{str(index).ljust(5)}{stream_handler.format(log)}")



#!/usr/bin/env python3
from dataclasses import dataclass
import enum
import logging, logging.config
import os
import sys
import pathlib
from typing import List


class LogColors(enum.Enum):
    RED = "\033[0;31m"
    RED_BOLD = "\033[1;31m"
    GREEN = "\033[0;32m"
    GREEN_BOLD = "\033[1;32m"
    YELLOW = "\033[0;33m"
    YELLOW_BOLD = "\033[1;33m"
    BLUE = "\033[0;34m"
    BLUE_BOLD = "\033[1;34m"
    MAGENTA = "\033[0;35m"
    MAGENTA_BOLD = "\033[1;35m"
    CYAN = "\033[0;36m"
    CYAN_BOLD = "\033[1;36m"
    WHITE = "\033[0;37m"
    WHITE_BOLD = "\033[1;37m"
    GREY = "\033[0;38m"
    GREY_BOLD = "\033[1;38m"
    NONE = "\033[0m"


class StreamHandlerFormatter(logging.Formatter):
    """Custom stream handler (use colors)"""

    def __init__(self, fmt):
        super().__init__()
        self.fmt = fmt
        self.formats = {
            logging.DEBUG: LogColors.MAGENTA.value + self.fmt + LogColors.NONE.value,
            logging.INFO: LogColors.GREY.value + self.fmt + LogColors.NONE.value,
            25 : LogColors.GREEN.value + self.fmt + LogColors.NONE.value,
            logging.WARNING: LogColors.YELLOW.value + self.fmt + LogColors.NONE.value,
            logging.ERROR: LogColors.RED.value + self.fmt + LogColors.NONE.value,
            logging.CRITICAL: LogColors.RED_BOLD.value + self.fmt + LogColors.NONE.value,
        }

    def format(self, record):
        log_fmt = self.fmt
        if record.levelno in self.formats:
            log_fmt = self.formats.get(record.levelno)
        formatter = logging.Formatter(log_fmt)

        return formatter.format(record)


@dataclass
class Logger:
    """Custom Logger"""

    log_configuration_path: str = "" #@TODO default value from config

    def __post_init__(self):
        # Check if configuration file exists
        if self.log_configuration_path:
            conf = pathlib.Path(self.log_configuration_path).resolve()
            if conf.exists():
                self.log_configuration_path = str(conf)
            else:
                print(f"configuration not found in '{self.log_configuration_path}'")
                self.log_configuration_path = ""

        # Load default configuration if needed
        if not self.log_configuration_path:
            conf = pathlib.Path(f"{os.path.dirname(__file__)}/default_logger.cfg").resolve()
            if conf.exists():
                self.log_configuration_path = str(conf)

        if self.log_configuration_path:
            logging.addLevelName(25, "OK")
            # Load configuration
            logging.config.fileConfig(self.log_configuration_path)
            self._logger = logging.getLogger("default")

            # Format the logger handlers
            for handler in self._logger.handlers:
                if isinstance(handler, logging.FileHandler):
                    pass
                elif isinstance(handler, logging.StreamHandler) and handler.formatter:
                    handler.setFormatter(StreamHandlerFormatter(handler.formatter._fmt))

            self._logger.debug(f"LAUNCH : {sys.argv}")
        else:
            exit()

    def get_streamHandlers(self):
        handlers: List[logging.StreamHandler] = []
        for handler in self._logger.handlers:
            if isinstance(handler, logging.FileHandler):
                pass
            elif isinstance(handler, logging.StreamHandler):
                handlers.append(handler)

        return handlers

    def get_fileHandlers(self):
        handlers: List[logging.FileHandler] = []
        for handler in self._logger.handlers:
            if isinstance(handler, logging.FileHandler):
                handlers.append(handler)

        return handlers

    def set_stream_handlers_level(self, level: int):
        for handler in self._logger.handlers:
            if isinstance(handler, logging.FileHandler):
                pass
            elif isinstance(handler, logging.StreamHandler):
                handler.setLevel(level)

    def debug(self, message: str, *args, **kws):
        self._logger._log(logging.DEBUG, message, args, **kws)

    def info(self, message: str, *args, **kws):
        self._logger._log(logging.INFO, message, args, **kws)

    def ok(self, message : str, *args, **kws):
        self._logger._log(25, message, args, **kws)

    def warning(self, message: str, *args, **kws):
        self._logger._log(logging.WARNING, message, args, **kws)

    def error(self, message: str, *args, **kws):
        self._logger._log(logging.ERROR, message, args, **kws)

    def critical(self, message: str, *args, **kws):
        self._logger._log(logging.CRITICAL, message, args, **kws)
